@@ -1,9 +1,7 @@
-/*	$NetBSD: supcmain.c,v 1.23 2007/07/20 18:58:14 christos Exp $	*/
-
 /*
  * Copyright (c) 1992 Carnegie Mellon University
  * All Rights Reserved.
- *
+ * 
  * Permission to use, copy, modify and distribute this software and its
  * documentation is hereby granted, provided that both the copyright
  * notice and this permission notice appear in all copies of the
@@ -16,7 +14,7 @@
  *
  * Carnegie Mellon requests users of this software to return to
  *
- *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
+ *  Software Distribution Coordinator  or  Software_Distribution@CS.CMU.EDU
  *  School of Computer Science
  *  Carnegie Mellon University
  *  Pittsburgh PA 15213-3890
@@ -45,11 +43,6 @@
  *			that each collection was last upgraded, rather than
  *			performing actual upgrades.
  *
- *	-u	"utimes" flag
- *			When this flags is given sup will not attempt to
- *			restore access and modification files on the collection
- *			files from the server.
- *
  *	-R	"resource pause" flag
  *			Sup will not disable resource pausing and will not
  *			make filesystem space checks.
@@ -180,20 +173,39 @@
  *	Modified SUP to use gzip based compression when sending files
  *	across the network to save BandWidth
  *
+ * $Log: supcmain.c,v $
+ * Revision 1.2  1994/08/11  02:46:22  rich
+ * Added extensions written by David Dawes.  From the man page:
+ *
+ * The -u flag, or the noupdate supfile option prevent updates from
+ * occurring for regular files where the modification time and mode
+ * hasn't changed.
+ *
+ * Now, how do we feed these patches back to CMU for consideration?
+ *
+ * Revision 1.1.1.1  1993/08/21  00:46:34  jkh
+ * Current sup with compression support.
+ *
+ * Revision 1.2  1993/05/24  17:57:28  brezak
+ * Remove netcrypt.c. Remove unneeded files. Cleanup make.
+ *
+ * Revision 1.1.1.1  1993/05/21  14:52:18  cgd
+ * initial import of CMU's SUP to NetBSD
+ *
  * Revision 1.6  92/08/11  12:06:59  mrt
  * 	Merged in Brad's changes. Made resource pausing code conditional
  * 	on MACH, rather than CMUCS. Fixed some calls to sprintf to
  * 	return void.
  * 	[92/08/09            mrt]
- *
+ * 
  * Revision 1.5  92/02/08  19:01:18  mja
  * 	Correct oldsigsys type when ANSI C.
  * 	[92/02/08  18:59:47  mja]
- *
+ * 
  * Revision 1.4  92/02/08  18:24:01  mja
  * 	Added -k and -K switches.
  * 	[92/01/17            vdelvecc]
- *
+ * 
  * 27-Dec-87  Glenn Marcy (gm0w) at Carnegie-Mellon University
  *	Added crosspatch support (is currently ignored).
  *
@@ -248,7 +260,7 @@
  *
  * 03-Feb-86  Glenn Marcy (gm0w) at Carnegie-Mellon University
  *	Fixed small bug in signon that didn't retry connections if an
- *	error occurred on the first attempt to connect. [V5.12]
+ *	error occured on the first attempt to connect. [V5.12]
  *
  * 26-Jan-86  Glenn Marcy (gm0w) at Carnegie-Mellon University
  *	New command interface.  Added -bBDEO flags and "delete",
@@ -272,7 +284,7 @@
  *	the user and group specified by the remote system.  Changed
  *	execute code to invalidate collection if execute command returns
  *	with a non-zero exit status.  Added support for execv() of
- *	original arguments of supfile is upgraded successfully.  Changed
+ *	original arguments of supfile is upgraded sucessfully.  Changed
  *	copyfile to always use a temp file if possible. [V5.7]
  *
  * 04-Jan-86  Glenn Marcy (gm0w) at Carnegie-Mellon University
@@ -322,61 +334,54 @@
 #define SYS_rpause	(-5)
 #endif
 #endif
-#include "supextern.h"
 
 /*********************************************
  ***    G L O B A L   V A R I A B L E S    ***
  *********************************************/
 
-char program[] = "SUP";		/* program name for SCM messages */
-int progpid = -1;		/* and process id */
+char program[] = "SUP";			/* program name for SCM messages */
+int progpid = -1;			/* and process id */
 
-COLLECTION *firstC, *thisC;	/* collection list pointer */
+COLLECTION *firstC,*thisC;		/* collection list pointer */
 
-extern int dontjump;		/* disable longjmp */
-extern int scmdebug;		/* SCM debugging flag */
+extern int dontjump;			/* disable longjmp */
+extern int scmdebug;			/* SCM debugging flag */
 
-int silent;			/* Silent run, print only errors */
-int sysflag;			/* system upgrade flag */
-int timeflag;			/* print times flag */
-int noutime;			/* Don't preserve utimes */
+int sysflag;				/* system upgrade flag */
+int timeflag;				/* print times flag */
 #if	MACH
-int rpauseflag;			/* don't disable resource pausing */
-#endif				/* MACH */
-int xpatchflag;			/* crosspatched with remote system */
-int portdebug;			/* network debugging ports */
-
-int main(int, char **);
-static int checkcoll(TREE *, void *);
-static void doswitch(char *, TREE **, int *, int *);
-static char *init(int, char **);
+int rpauseflag;				/* don't disable resource pausing */
+#endif	/* MACH */
+int xpatchflag;				/* crosspatched with remote system */
+int portdebug;				/* network debugging ports */
 
 /*************************************
  ***    M A I N   R O U T I N E    ***
  *************************************/
 
-int
-main(int argc, char **argv)
+main (argc,argv)
+int argc;
+char **argv;
 {
-	char *progname, *supfname;
-	int restart, sfdev = 0, sfino = 0;
-	time_t sfmtime = 0;
+	char *init ();
+	char *progname,*supfname;
+	int restart,sfdev,sfino,sfmtime;
 	struct stat sbuf;
-	struct sigaction ign;
+	struct sigvec ignvec,oldvec;
 
 	/* initialize global variables */
-	pgmversion = PGMVERSION;/* export version number */
-	server = FALSE;		/* export that we're not a server */
-	collname = NULL;	/* no current collection yet */
-	dontjump = TRUE;	/* clear setjmp buffer */
-	progname = estrdup(argv[0]);
+	pgmversion = PGMVERSION;	/* export version number */
+	server = FALSE;			/* export that we're not a server */
+	collname = NULL;		/* no current collection yet */
+	dontjump = TRUE;		/* clear setjmp buffer */
+	progname = salloc (argv[0]);
 
-	supfname = init(argc, argv);
-	restart = -1;		/* don't make restart checks */
+	supfname = init (argc,argv);
+	restart = -1;			/* don't make restart checks */
 	if (*progname == '/' && *supfname == '/') {
-		if (stat(supfname, &sbuf) < 0) {
-			logerr("Can't stat supfile %s", supfname);
-		} else {
+		if (stat (supfname,&sbuf) < 0)
+			logerr ("Can't stat supfile %s",supfname);
+		else {
 			sfdev = sbuf.st_dev;
 			sfino = sbuf.st_ino;
 			sfmtime = sbuf.st_mtime;
@@ -385,66 +390,57 @@ main(int argc, char **argv)
 	}
 	if (timeflag) {
 		for (thisC = firstC; thisC; thisC = thisC->Cnext)
-			prtime();
+			prtime ();
 	} else {
 		/* ignore network pipe signals */
-		ign.sa_handler = SIG_IGN;
-		ign.sa_flags = 0;
-		sigemptyset(&ign.sa_mask);
-		(void) sigaction(SIGPIPE, &ign, NULL);
-		getnams();	/* find unknown repositories */
+		ignvec.sv_handler = SIG_IGN;
+		ignvec.sv_onstack = 0;
+		ignvec.sv_mask = 0;
+		(void) sigvec (SIGPIPE,&ignvec,&oldvec);
+		getnams ();		/* find unknown repositories */
 		for (thisC = firstC; thisC; thisC = thisC->Cnext) {
-			getcoll();	/* upgrade each collection */
+			getcoll ();	/* upgrade each collection */
 			if (restart == 0) {
-				if (stat(supfname, &sbuf) < 0)
-					logerr("Can't stat supfile %s",
-					    supfname);
+				if (stat (supfname,&sbuf) < 0)
+					logerr ("Can't stat supfile %s",
+						supfname);
 				else if (sfmtime != sbuf.st_mtime ||
-#ifndef __CYGWIN__	/* Cygwin's inodes are not constant */
-					    sfino != sbuf.st_ino ||
-#endif
-				    sfdev != sbuf.st_dev) {
+					 sfino != sbuf.st_ino ||
+					 sfdev != sbuf.st_dev) {
 					restart = 1;
 					break;
 				}
 			}
 		}
-		endpwent();	/* close /etc/passwd */
-		(void) endgrent();	/* close /etc/group */
+		endpwent ();		/* close /etc/passwd */
+		(void) endgrent ();	/* close /etc/group */
 		if (restart == 1) {
 			int fd;
-			if (!silent)
-				loginfo("SUP Restarting %s with new supfile %s",
-				    progname, supfname);
-			for (fd = getdtablesize(); fd > 3; fd--)
-				(void) close(fd);
-			execv(progname, argv);
-			logquit(1, "Restart failed");
+			loginfo ("SUP Restarting %s with new supfile %s",
+				progname,supfname);
+			for (fd = getdtablesize (); fd > 3; fd--)
+				(void) close (fd);
+			execv (progname,argv);
+			logquit (1,"Restart failed");
 		}
 	}
-	while ((thisC = firstC) != NULL) {
+	while (thisC = firstC) {
 		firstC = firstC->Cnext;
-		free(thisC->Cname);
-		Tfree(&thisC->Chtree);
-		free(thisC->Cbase);
-		if (thisC->Chbase)
-			free(thisC->Chbase);
-		if (thisC->Cprefix)
-			free(thisC->Cprefix);
-		if (thisC->Crelease)
-			free(thisC->Crelease);
-		if (thisC->Cnotify)
-			free(thisC->Cnotify);
-		if (thisC->Clogin)
-			free(thisC->Clogin);
-		if (thisC->Cpswd)
-			free(thisC->Cpswd);
-		if (thisC->Ccrypt)
-			free(thisC->Ccrypt);
-		free(thisC);
+		free (thisC->Cname);
+		Tfree (&thisC->Chtree);
+		free (thisC->Cbase);
+		if (thisC->Chbase)  free (thisC->Chbase);
+		if (thisC->Cprefix)  free (thisC->Cprefix);
+		if (thisC->Crelease)  free (thisC->Crelease);
+		if (thisC->Cnotify)  free (thisC->Cnotify);
+		if (thisC->Clogin)  free (thisC->Clogin);
+		if (thisC->Cpswd)  free (thisC->Cpswd);
+		if (thisC->Ccrypt)  free (thisC->Ccrypt);
+		free ((char *)thisC);
 	}
-	exit(0);
+	exit (0);
 }
+
 /*****************************************
  ***    I N I T I A L I Z A T I O N    ***
  *****************************************/
@@ -457,18 +453,20 @@ main(int argc, char **argv)
 #define Twant	Tuid
 #define Tcount	Tgid
 
-static void 
-doswitch(char *argp, TREE ** collTp, int *oflagsp, int *aflagsp)
+doswitch (argp,collTp,oflagsp,aflagsp)
+char *argp;
+register TREE **collTp;
+int *oflagsp,*aflagsp;
 {
-	TREE *t;
-	char *coll;
-	int oflags, aflags;
+	register TREE *t;
+	register char *coll;
+	register int oflags,aflags;
 
 	oflags = aflags = 0;
 	for (;;) {
 		switch (*argp) {
 		default:
-			logerr("Invalid flag '%c' ignored", *argp);
+			logerr ("Invalid flag '%c' ignored",*argp);
 			break;
 		case '\0':
 		case '=':
@@ -480,13 +478,13 @@ doswitch(char *argp, TREE ** collTp, int *oflagsp, int *aflagsp)
 				return;
 			}
 			do {
-				coll = nxtarg(&argp, ", \t");
-				t = Tinsert(collTp, coll, TRUE);
+				coll = nxtarg (&argp,", \t");
+				t = Tinsert (collTp,coll,TRUE);
 				t->Toflags |= oflags;
 				t->Toflags &= ~aflags;
 				t->Taflags |= aflags;
 				t->Taflags &= ~oflags;
-				argp = skipover(argp, ", \t");
+				argp = skipover (argp,", \t");
 			} while (*argp);
 			return;
 		case 'N':
@@ -498,14 +496,11 @@ doswitch(char *argp, TREE ** collTp, int *oflagsp, int *aflagsp)
 		case 'R':
 #if	MACH
 			rpauseflag = TRUE;
-#endif				/* MACH */
+#endif	/* MACH */
 			break;
 		case 'X':
 			xpatchflag = TRUE;
 			break;
-		case 'S':
-			silent = TRUE;
-			break;
 		case 's':
 			sysflag = TRUE;
 			break;
@@ -565,175 +560,177 @@ doswitch(char *argp, TREE ** collTp, int *oflagsp, int *aflagsp)
 			aflags |= CFOLD;
 			break;
 		case 'u':
-			noutime = TRUE;
+			oflags |= CFNOUPDATE;
+			aflags &= ~CFNOUPDATE;
+			break;
+		case 'U':
+			oflags &= ~CFNOUPDATE;
+			aflags |= CFNOUPDATE;
 			break;
 		case 'v':
 			oflags |= CFVERBOSE;
 			break;
 		case 'z':
 			oflags |= CFCOMPRESS;
+			aflags &= ~CFCOMPRESS;
 			break;
 		case 'Z':
 			oflags &= ~CFCOMPRESS;
+			aflags |= CFCOMPRESS;
 			break;
 		}
 		argp++;
 	}
 }
 
-static char *
-init(int argc, char **argv)
+char *init (argc,argv)
+int argc;
+char **argv;
 {
-	char buf[STRINGLENGTH], *p;
-	const char *u;
+	char buf[STRINGLENGTH],*p;
 	char username[STRINGLENGTH];
-	char *supfname, *arg;
-	COLLECTION *c, *lastC;
-	FILE *f;
-	int bogus;
-	struct passwd *pw;
-	TREE *t;
-	TREE *collT;		/* collections we are interested in */
-	time_t timenow;		/* startup time */
-	int oflags, aflags;
+	register char *supfname,*q,*arg;
+	register COLLECTION *c,*lastC;
+	register FILE *f;
+	register int bogus;
+	register struct passwd *pw;
+	register TREE *t;
+	TREE *collT;			/* collections we are interested in */
+	long timenow;			/* startup time */
+	int checkcoll ();
+	int oflags,aflags;
 	int cwant;
 #ifdef	MACH
-	void (*oldsigsys) ();
-#endif				/* MACH */
+#ifdef	__STDC__
+	void (*oldsigsys)();
+#else
+	int (*oldsigsys)();
+#endif
+#endif /* MACH */
+	char *fmttime();
 
-	sysflag = FALSE;	/* not system upgrade */
-	timeflag = FALSE;	/* don't print times */
+	sysflag = FALSE;		/* not system upgrade */
+	timeflag = FALSE;		/* don't print times */
 #if	MACH
-	rpauseflag = FALSE;	/* don't disable resource pausing */
-#endif				/* MACH */
-	xpatchflag = FALSE;	/* not normally crosspatched */
-	scmdebug = 0;		/* level zero, no SCM debugging */
-	portdebug = FALSE;	/* no debugging ports */
+	rpauseflag = FALSE;		/* don't disable resource pausing */
+#endif	/* MACH */
+	xpatchflag = FALSE;		/* not normally crosspatched */
+	scmdebug = 0;			/* level zero, no SCM debugging */
+	portdebug = FALSE;		/* no debugging ports */
 
 	collT = NULL;
 	oflags = aflags = 0;
 	while (argc > 1 && argv[1][0] == '-' && argv[1][1] != '\0') {
-		doswitch(&argv[1][1], &collT, &oflags, &aflags);
+		doswitch (&argv[1][1],&collT,&oflags,&aflags);
 		--argc;
 		argv++;
 	}
 	if (argc == 1 && !sysflag)
-		logquit(1, "Need either -s or supfile");
+		logquit (1,"Need either -s or supfile");
 #if	MACH
-	oldsigsys = signal(SIGSYS, SIG_IGN);
+	oldsigsys = signal (SIGSYS,SIG_IGN);
 	if (rpauseflag != TRUE)
-		if (syscall(SYS_rpause, ENOSPC, RPAUSE_ALL, RPAUSE_DISABLE) < 0)
-			rpauseflag = TRUE;
-	(void) signal(SIGSYS, oldsigsys);
-#endif				/* MACH */
+		if (syscall (SYS_rpause,ENOSPC,RPAUSE_ALL,RPAUSE_DISABLE) < 0)
+		rpauseflag = TRUE;
+	(void) signal (SIGSYS,oldsigsys);
+#endif	/* MACH */
 	if (sysflag)
-		(void) sprintf(supfname = buf,
-		    timeflag ? FILESUPTDEFAULT : FILESUPDEFAULT,
-		    DEFDIR);
+		(void) sprintf (supfname = buf,
+				    timeflag?FILESUPTDEFAULT:FILESUPDEFAULT,
+				    DEFDIR);
 	else {
 		supfname = argv[1];
-		if (strcmp(supfname, "-") == 0)
+		if (strcmp (supfname,"-") == 0)
 			supfname = "";
 		--argc;
 		argv++;
 	}
 	cwant = argc > 1;
 	while (argc > 1) {
-		t = Tinsert(&collT, argv[1], TRUE);
+		t = Tinsert (&collT,argv[1],TRUE);
 		t->Twant = TRUE;
 		--argc;
 		argv++;
 	}
-	if ((u = getlogin()) ||
-	    ((pw = getpwuid((int) getuid())) && (u = pw->pw_name)))
-		(void) strcpy(username, u);
+	if ((p = (char *)getlogin()) ||
+	    ((pw = getpwuid ((int)getuid())) && (p = pw->pw_name)))
+		(void) strcpy (username,p);
 	else
 		*username = '\0';
 	if (*supfname) {
-		f = fopen(supfname, "r");
+		f = fopen (supfname,"r");
 		if (f == NULL)
-			logquit(1, "Can't open supfile %s", supfname);
+			logquit (1,"Can't open supfile %s",supfname);
 	} else
 		f = stdin;
 	firstC = NULL;
 	lastC = NULL;
 	bogus = FALSE;
-	while ((p = read_line(f, NULL, NULL, NULL, 0)) != NULL) {
-		if (index("#;:", *p))
-			continue;
-		arg = nxtarg(&p, " \t");
+	while (p = fgets (buf,STRINGLENGTH,f)) {
+		q = index (p,'\n');
+		if (q)  *q = '\0';
+		if (index ("#;:",*p))  continue;
+		arg = nxtarg (&p," \t");
 		if (*arg == '\0') {
-			logerr("Missing collection name in supfile");
+			logerr ("Missing collection name in supfile");
 			bogus = TRUE;
 			continue;
 		}
 		if (cwant) {
-			TREE *mytree;
-			if ((mytree = Tsearch(collT, arg)) == NULL)
+			register TREE *t;
+			if ((t = Tsearch (collT,arg)) == NULL)
 				continue;
-			mytree->Tcount++;
+			t->Tcount++;
 		}
-		c = (COLLECTION *) malloc(sizeof(COLLECTION));
-		if (c == NULL) {
-			logerr("Cannot allocate memory");
-			exit(1);
-		}
-		if (firstC == NULL)
-			firstC = c;
-		if (lastC != NULL)
-			lastC->Cnext = c;
+		c = (COLLECTION *) malloc (sizeof(COLLECTION));
+		if (firstC == NULL)  firstC = c;
+		if (lastC != NULL) lastC->Cnext = c;
 		lastC = c;
-		if (parsecoll(c, arg, p) < 0) {
+		if (parsecoll(c,arg,p) < 0) {
 			bogus = TRUE;
 			continue;
 		}
 		c->Cflags |= oflags;
 		c->Cflags &= ~aflags;
-		if ((t = Tsearch(collT, c->Cname)) != NULL) {
+		if (t = Tsearch (collT,c->Cname)) {
 			c->Cflags |= t->Toflags;
 			c->Cflags &= ~t->Taflags;
 		}
-		if ((c->Cflags & CFMAIL) && c->Cnotify == NULL) {
+		if ((c->Cflags&CFMAIL) && c->Cnotify == NULL) {
 			if (*username == '\0')
-				logerr("User unknown, notification disabled");
+				logerr ("User unknown, notification disabled");
 			else
-				c->Cnotify = estrdup(username);
+				c->Cnotify = salloc (username);
 		}
 		if (c->Cbase == NULL) {
-			(void) sprintf(buf, FILEBASEDEFAULT, c->Cname);
-			c->Cbase = estrdup(buf);
+			(void) sprintf (buf,FILEBASEDEFAULT,c->Cname);
+			c->Cbase = salloc (buf);
 		}
 	}
-	if (bogus)
-		logquit(1, "Aborted due to supfile errors");
-	if (f != stdin)
-		(void) fclose(f);
-	if (cwant)
-		(void) Tprocess(collT, checkcoll, NULL);
-	Tfree(&collT);
-	if (firstC == NULL)
-		logquit(1, "No collections to upgrade");
-	timenow = time((time_t *) NULL);
+	if (bogus)  logquit (1,"Aborted due to supfile errors");
+	if (f != stdin)  (void) fclose (f);
+	if (cwant)  (void) Tprocess (collT,checkcoll);
+	Tfree (&collT);
+	if (firstC == NULL)  logquit (1,"No collections to upgrade");
+	timenow = time ((long *)NULL);
 	if (*supfname == '\0')
 		p = "standard input";
 	else if (sysflag)
 		p = "system software";
 	else
-		(void) sprintf(p = buf, "file %s", supfname);
-	if (!silent)
-		loginfo("SUP %d.%d (%s) for %s at %s", PROTOVERSION, PGMVERSION,
-		    scmversion, p, fmttime(timenow));
-	return (estrdup(supfname));
+		(void) sprintf (p = buf,"file %s",supfname);
+	loginfo ("SUP %d.%d (%s) for %s at %s",PROTOVERSION,PGMVERSION,
+		scmversion,p,fmttime (timenow));
+	return (salloc (supfname));
 }
 
-static int
-checkcoll(TREE * t, void *dummy __unused)
+checkcoll (t)
+register TREE *t;
 {
-	if (!t->Twant)
-		return (SCMOK);
+	if (!t->Twant)  return (SCMOK);
 	if (t->Tcount == 0)
-		logerr("Collection %s not found", t->Tname);
+		logerr ("Collection %s not found",t->Tname);
 	if (t->Tcount > 1)
-		logerr("Collection %s found more than once", t->Tname);
+		logerr ("Collection %s found more than once",t->Tname);
 	return (SCMOK);
 }
